# -*- coding: ISO-8859-1 -*-
""" capellaScript -- 
>>>Partitur-Synthese (Version 5.0) 
Zusammenfgen von Einzelstimmen oder| Einzelpartituren zu einer 
Gesamtpartitur||
Mehrtaktpausen und Taktdifferenzen zwischen den jeweils zusammenzufgenden Systemen werden erkannt und angezeigt.||
VORGEHENSWEISE:|
Bitte beachten Sie die detaillierten Informationen unter "Dokumentation" (unten anklicken).|
Die Dokumentationsdatei "Partitur-Synthese.chm" muss dafr zustzlich zum Python-Skript "Partitur-Synthese.py"
 in den "scripts"-Ordner des capella-Verzeichnisses bertragen werden.||
Rckmeldungen bitte an Hans H. Lampe:|HansHermann.Lampe@t-online.de
<<<
ab Version 2.0: Erkennen und Anzeigen von Mehrtaktpausen
ab Version 3.0: Erkennen und Anzeigen von Notenwert-Differenzen in zu vereinigenden Systemen
ab Version 4.0: Erkennen und Anzeigen von Mehrtaktpausen in der ersten Einzelstimme/Partitur
                Bei 'Abbrechen' verlassen des Skripts
ab Version 5.0: verbesserte Diagnose bei unterschiedlicher Anzahl von Systemen in den Einzelstimmen
"""

import xml.dom.minidom
DOC = xml.dom.minidom.Document

 
# ---- Dialogbox zur Eingabe des Dateinamens der neuen Einzelpartitur ---- #

def dialog():
    global fileName, tempInput2, dlg, edit
    edit = Edit('', width=24)
    dlg = Dialog('Einzel-Stimme /-Partitur', edit)
        
# ---- Prfung auf Mehrtaktpausen ---- #

def multipleRests(score):
    global exit
    exit=0 # fr Abfrage und eventuelles Beenden des Skripts #
    syscnt = 0
    for system in score.getElementsByTagName('system'):
        syscnt += 1
        for rest in system.getElementsByTagName('rest'):
            duration = rest.getElementsByTagName('duration')[0]
            base = duration.getAttribute('base')
            length = len(base)
            if length > 1:
                if base[1] != '/': # z.B. 1/2, 1/4, usw. #
                    intbase = int(base)
                    if base > '1': # => 10 #
                        activeScore().read(tempInput2) # die letzte Einzelstimme/Partitur
                        messageBox('Mehrtaktpause in "'+str(fileName)+'" vorhanden',
                                   'Bitte Mehrtaktpause im '+str(syscnt)+
                                   '. System als Ganztaktpausen auflsen!')
                        activeScore().read(tempInput1) # die bisherige Partitur
                        exit=1  # fr Abfrage und eventuelles Beenden des Skripts #
            else: # Lnge = 1 #
                if base > '1': # 2 - 9 #
                    activeScore().read(tempInput2) # die letzte Einzelstimme/Partitur
                    messageBox('Mehrtaktpause in "'+str(fileName)+'" vorhanden', 'Bitte Mehrtaktpause(n) als Ganztaktpausen auflsen!')
                    activeScore().read(tempInput1) # die bisherige Partitur
                    exit=1  # fr Abfrage und eventuelles Beenden des Skripts #

# ---- Vergleich der Systeme ---- #

def compareSystems(score):
    global exit
    exit = 0
    d = 0
    dd = [0]
    e = 0
    ee = [0]
    i = 0

    for system in score.systems(): # Bestimmen der Notenwerte in den Systemen der aktiven Partitur '
        for obj in system.staff(0).voice(0).noteObjs():
            d += obj.duration()
        dd.append(0)
        dd[i] = d
        d = 0
        i = i + 1
        
    activeScore().read(tempInput2) # Die neue Einzelpartitur wird zur aktiven Partitur #

    i = 0
    for system in score.systems(): # Bestimmen der Notenwerte in den Systemen der neuen Einzelpartitur #
        for obj in system.staff(0).voice(0).noteObjs():
            e += obj.duration()
        ee.append(0)
        ee[i] = e
        e = 0
        i = i + 1

    i = 0
    for d in dd: # Vergleichen der Notenwerte in den entsprechenden Systemen #
        if dd[i] == ee[i]:
            if dd[i] != 0:
                i = i + 1
            
        else:
            if dd[i] == 0: # neue Einzelstimme hat mehr Systeme #
                return
            if ee[i] == 0: # neue Einzelstimme hat weniger Systeme #
                return

            else:
                messageBox('Notenwert-Differenz in "'+str(fileName)+'"', 'Der Notengesamtwert im '
                           +str(i+1)+'. Notensystem ist '+str(ee[i])+' anstatt '+str(dd[i]))
                activeScore().read(tempInput1) # Wiederherstellen der bisherigen Partitur #
                exit = 1
                return

# ---- Analyse der neuen Einzelpartitur ---- #

def prepare(score):
    global newStaffLayouts, newSystems, iSavePr
    
    # ---- Mustersystem ---- #

    newStaffLayouts = score.getElementsByTagName('staffLayout')
   
    # ---- Partitur ---- #
    
    i = 0        
    newSystems = []
    for system in score.getElementsByTagName('system'):
            newSystems.append('0')
            newSystems[i] = system
            iSavePr = i
            i = i + 1


# ---- Zusammenfgen von aktueller Partitur und neuer Einzelpartitur ---- # 

def merge(score):
    global iSaveM
    
    # ---- Das Mustersystem ---- #

    description = 1
    staffLayouts = score.getElementsByTagName('staffLayout')
    
    for staffLayout in staffLayouts:

        # Eindeutige numerische "Beschreibung" der Notenzeilen im bisherigen Mustersystem #
        staffLayout.setAttribute('description',str(description))
        description = description + 1
        
    for newStaffLayout in newStaffLayouts:
            staffLayout.parentNode.appendChild(newStaffLayout)

            # Eindeutige numerische "Beschreibung" der Notenzeilen in der Mustersystem-Ergnzung #
            newStaffLayout.setAttribute('description',str(description))
            description = description + 1


    # ---- Die Partitur ---- #

    i=0
    
    for system in score.getElementsByTagName('system'):
        iSaveM = i
        description = 1
        if i > iSavePr:            
            return 
        else:
            staves = system.getElementsByTagName('staves')[0]
            staffs = staves.getElementsByTagName('staff')
            for staff in staffs:

                # Eindeutige Zuordnung der bisherigen Partiturzeilen zu den Notenzeilen des Mustersystems #
                staff.setAttribute('layout',str(description))
                description = description + 1

                newSystem = newSystems[i]
                newStaves = newSystem.getElementsByTagName('staves')[0]

            for newStaffs in newStaves.getElementsByTagName('staff'):
                staff.parentNode.appendChild(newStaffs)

                # Eindeutige Zuordnung der neuen Partiturzeilen zu den Notenzeilen der Mustersystem-Ergnzung #
                newStaffs.setAttribute('layout',str(description))
                description = description + 1

            i = i + 1  # Das nchste System #
            
                            
# Hauptprogramm:

from caplib.capDOM import ScoreChange
import tempfile, codecs

class newScoreChange1(ScoreChange): # Analyse der neuen Einzelpartitur # 
    def changeScore(self, score):
        global scriptAction, doc
        doc = score.parentNode
        prepare(score)

class newScoreChange2(ScoreChange): # Zusammenfgen von aktueller Partitur und neuer Einzelpartitur #
    def changeScore(self, score):
        global scriptAction, doc
        doc = score.parentNode
        merge(score)

class newScoreChange3(ScoreChange): # Test auf Mehrfachpause #
    def changeScore(self, score):
        global scriptAction, doc
        doc = score.parentNode
        multipleRests(score)


if activeScore():
        
    activeScore().registerUndo("Partitur-Synthese")
    tempInput1 = tempfile.mktemp('.capx') # aktuelle aktive Partitur #
    tempInput2 = tempfile.mktemp('.capx') # neue Einzelpartitur #
    tempOutput = tempfile.mktemp('.capx')

    activeScore().write(tempInput1) # erste Einzelstimme/Partitur nach Input1 #
    activeScore().write(tempInput2) # erste Einzelstimme/Partitur nach Input2 fr Test auf Mehrtaktpausen #
    fileName = activeScore().title() # Fr die Anzeige des aktuellen Datei-Namens bei Entdecken von Mehrfachpausen #
    
    newScoreChange3(tempInput1, tempOutput) # Test auf Mehrtaktpausen in der ersten Partitur #
    
    if exit == 0: # Abfrage des Indikators fr Mehrtaktpausen #
        dialog() # ---- Aufrufen der Dialogbox zur Eingabe des Partitur-Dateinamens ---- #
        if dlg.run(): # nur bei OK - nicht bei "Abbrechen"
            fileName = edit.value()
            tempInput2 = 'c:\\Einzelstimmen\\' + str(fileName) + '.capx' # Aufruf der neuen Einzelpartitur #
                      
            newScoreChange3(tempInput2, tempOutput) # Test auf Mehrtaktpausen in nachfolgender Einzelpartitur #
            if exit ==0: # Abfrage des Indikators fr Mehrtaktpausen #
                newScoreChange1(tempInput2, tempOutput) # Analyse der neuen Einzelpartitur #

                compareSystems(activeScore()) # Vergleich der einzelnen System der bisherigen mit der neuen Partitur #
                if exit ==0:
                    newScoreChange2(tempInput1, tempOutput) # Zusammenfgen von aktueller Partitur und neuer Einzelpartitur #
                    if iSavePr < iSaveM: # Vergleich der Anzahl der Systeme #
                        messageBox('        Anzahl der Systeme zu klein','Die neue Einzelstimme "'
                                   +str(fileName)+'" hat nur '+str(iSavePr+1)+' Systeme anstatt '+str(iSaveM+1))
                        activeScore().read(tempInput1) # Wiederherstellen der bisherigen Partitur #
                        os.remove(tempInput1)
                        os.remove(tempOutput)       
                    else:
                        if iSavePr > iSaveM: # Vergleich der Anzahl der Systeme #
                            messageBox('        Anzahl der Systeme zu gro','Die neue Einzelstimme "'
                                       +str(fileName)+'" hat '+str(iSavePr+1)+' Systeme anstatt nur '+str(iSaveM+1))
                            activeScore().read(tempInput1) # Wiederherstellen der bisherigen Partitur #
                            os.remove(tempInput1)
                            os.remove(tempOutput)
                        else: # gleiche Anzahl von Systemen #
                            activeScore().read(tempOutput) #mit diesem Aufruf wird das Problem mit einer Grafik in einer "Sekundr"-Partitur offenbar!!!
                            os.remove(tempInput1)
                            # os.remove(tempInput2) # lscht, wenn aktiviert, fehlerhafterweise die Ursprungs-Datei!!! #
                            os.remove(tempOutput)
                else: # Taktdifferenz zwischen zu vereinigenden Systemen #
                    os.remove(tempInput1)
                    os.remove(tempOutput) 
            else: # Mehrtaktpause(n) in nachfolgender Einzelpartitur #
                    os.remove(tempInput1)
                    os.remove(tempOutput)
        else: # Mehrtaktpause(n) in erster Partitur # 
            os.remove(tempInput1)
            os.remove(tempOutput)   
                    
    
